%ODECOMPARE Comparisons of several ode solvers
%  PURPOSE
%     It compares several ode solvers
%     In particular:
%        o The Euler's method
%        o The Heun's method
%        o The Runge-Kutta IV method.
%     It compute also the absolute stabilitation region.
%
%  See also ODEEULERD1, ODEHEUND1, ODERK4, NUMODE.

%  AUTHOR : M. Venturin


% CLEAR WORKSPACE
close all;
clear all;
clc;

% Define the computational grid
fbox = [-3 3 -3 3];
xmin = fbox(1); xmax = fbox(2);
ymin = fbox(3); ymax = fbox(4);
xstep = (xmax-xmin)/400; ystep = (ymax-ymin)/400;
[x,y] = meshgrid(xmin:xstep:xmax,ymin:ystep:ymax);

% Evaluate stability expression (RK4)
% Re(h lambda) = x(i) , Im(h lambda) = y(i)
expr = 'abs(1+(x+i*y)+0.5*(x+i*y).^2+1/6*(x+i*y).^3+1/24*(x+i*y).^4)<=1';
eval(['z=',expr,';']);
z1 = double(~z);

% Evaluate stability expression (HEUN)
% Re(h lambda) = x(i) , Im(h lambda) = y(i)
expr = 'abs(1+(x+i*y)+0.5*(x+i*y).^2)<=1';
eval(['z=',expr,';']);
z2 = double(~z);

% Evaluate stability expression (EULER)
% Re(h lambda) = x(i) , Im(h lambda) = y(i)
expr = 'abs(1+(x+i*y))<=1';
eval(['z=',expr,';']);
z3 = double(~z);

% Merge regions
z = (z1+z2+z3);

% Draw stability region
figure(1);
contourf(x,y,z,unique(z(:)));
colormap((gray+1)/2); grid off; axis square;
hold on
hf = line([xmin,xmax],[0,0]); set(hf,'Color','k');
hf = line([0,0],[ymin,ymax]); set(hf,'Color','k');
hold off
xlabel('Re(h\lambda)'); ylabel('Im(h\lambda)');
title('Absolute stability region');


% THE ERROR BEHAVIOUR
% The perform error estimation by dividing the step size h by 2 at every
% iteration.
[y0,tspan] = odeEq2I;   % standard conditions

hmax1 = zeros(6,1);    % Current step size
emax1 = zeros(6,1);    % Current error
h = 0.2;               % step h 
% perform error computation (Euler)
for k=1:length(hmax1)
   [te,ye] = odeEuler('odeEq2',tspan,y0,h);
   y = odeEq2S(te);
   emax1(k)=max(abs(y-ye));
   hmax1(k)=h;
   h = h/2;
end

hmax2 = zeros(6,1);    % Current step size
emax2 = zeros(6,1);    % Current error
h = 0.2;                % step h 
% perform error computation (Heun)
for k=1:length(hmax2)
   [te,ye] = odeHeun('odeEq2',tspan,y0,h);
   y = odeEq2S(te);
   emax2(k)=max(abs(y-ye));
   hmax2(k)=h;
   h = h/2;
end

hmax3 = zeros(6,1);    % Current step size
emax3 = zeros(6,1);    % Current error
h = 0.2;                % step h 
% perform error computation (Heun)
for k=1:length(hmax3)
   [te,ye] = odeRK4('odeEq2',tspan,y0,h);
   y = odeEq2S(te);
   emax3(k)=max(abs(y-ye));
   hmax3(k)=h;
   h = h/2;
end

% draw error
figure(2);
subplot(2,2,1);
odeErrorOp(hmax1,emax1,1);
legend('Euler',-1);
subplot(2,2,2);
odeErrorOp(hmax2,emax2,2);
legend('Heun',-1);
subplot(2,2,3);
odeErrorOp(hmax3,emax3,4);
legend('RK4',-1);
subplot(2,2,4);
loglog(hmax1,emax1,'k*-',hmax2,emax2,'bo-',hmax3,emax3,'r+-');
title(['Comparison results']);
axis square
legend('Euler','Heun','RK4',-1);
