function [t,y] = odeHeun(odefun,tspan,y0,h,varargin)
%ODEHEUN The Heun's method for ODE.
%  PURPOSE
%     It implements the Heun's method for ode.
%
%  SYNTAX
%     [t,y] = odeHeun(odefun,tspan,y0,h)
%     [t,y] = odeHeun(odefun,tspan,y0,h,P1,...)
%
%  INPUT
%     odefun   : The ode function
%     tspan    : The starting and ending simulation time
%     y0       : The initial condition
%     h        : The stepsize
%     P1       : Optional function parameter
%
%  OUTPUT
%     t        : The time solution vector
%     y        : The solution vector
%
%  See also ODEHEUND1, NUMODE.

%  AUTHOR : M. Venturin


t = (tspan(1):h:tspan(2))';   % allocate memory for t
n = length(t);                % number of elements
y = zeros(n,1);               % allocate memory for y
y(1) = y0;                    % initial condition

% main loop
for j=1:n-1
   % f(x_n,y_n)
   fxyn = feval(odefun,t(j),y(j),varargin{:});
   % update solution
   y(j+1) = y(j)+...
      0.5*h*(fxyn+feval(odefun,t(j+1),y(j)+h*fxyn,varargin{:}));
end
