function [t,y] = odeRK4(odefun,tspan,y0,h,varargin)
%ODERK4 The Runge-Kutta IV method for ODE.
%  PURPOSE
%     It implements the Runge-Kutta IV method for ode.
%
%  SYNTAX
%     [t,y] = odeRK4(odefun,tspan,y0,h)
%     [t,y] = odeRK4(odefun,tspan,y0,h,P1,...)
%
%  INPUT
%     odefun   : The ode function
%     tspan    : The starting and ending simulation time
%     y0       : The initial condition
%     h        : The stepsize
%     P1       : Optional function parameter
%
%  OUTPUT
%     t        : The time solution vector
%     y        : The solution vector
%
%  See also ODERK4D1, NUMODE.

%  AUTHOR : M. Venturin


t = (tspan(1):h:tspan(2))';   % allocate memory for t
n = length(t);                % number of elements
y = zeros(n,1);               % allocate memory for y
y(1) = y0;                    % initial condition

% main loop
for j=1:n-1
   k1 = h*feval(odefun,t(j),y(j),varargin{:});
   k2 = h*feval(odefun,t(j)+h/2,y(j)+k1/2,varargin{:});
   k3 = h*feval(odefun,t(j)+h/2,y(j)+k2/2,varargin{:});
   k4 = h*feval(odefun,t(j)+h,y(j)+k3,varargin{:});
   y(j+1) = y(j)+1/6*(k1+2*k2+2*k3+k4);
end
