%ODERK4D1 Demo for Heun's method.
%  PURPOSE
%     Demo for the Heun's method for the ODEEQ1 ode function.
%     In particular, we compute
%        o The absolute stability region
%        o The performace of the Euler's Method for three different steps
%        o The Heun's method applied to a complex equation
%        o The error behaviour.
%
%  See also ODEEULERD1, ODEHEUND1, ODECOMPARE, NUMODE.

%  AUTHOR : M. Venturin


% CLEAR WORKSPACE
close all;
clear all;
clc;


% THE ABSOLUTE STABILITY REGION
figure(1);
odeAbsComp([-3 3 -3 3],...
   'abs(1+(x+i*y)+0.5*(x+i*y).^2+1/6*(x+i*y).^3+1/24*(x+i*y).^4)<=1');


% THE RK4 METHOD (REAL)
[y0,tspan] = odeEq1I;   % standard conditions
lambda=-1;              % lambda 
h1 = 0.5;               % step h 
[t1,y1] = odeRK4('odeEq1',tspan,y0,h1,lambda);

h2 = 1.2;               % step h 
[t2,y2] = odeRK4('odeEq1',tspan,y0,h2,lambda);

h3 = 2.1;               % step h 
[t3,y3] = odeRK4('odeEq1',tspan,y0,h3,lambda);

% exact solution
t = linspace(tspan(1),tspan(2),100)';
y = odeEq1S(t,lambda);

% update stability region
hold on
hfig = plot(h1*lambda,0,'b*'); set(hfig,'MarkerSize',10);
hfig = plot(h2*lambda,0,'r*'); set(hfig,'MarkerSize',10);
hfig = plot(h3*lambda,0,'g*'); set(hfig,'MarkerSize',10);
hold off

% draw solutions
figure(2);
hfig = plot(t,y,'k-',t1,y1,'bo-',t2,y2,'r*-',t3,y3,'g+-');
set(hfig,'LineWidth',2);
legend('exact',['h=',num2str(h1)],['h=',num2str(h2)],['h=',num2str(h3)]);
title('Comparison results for Euler''s method');
xlabel('t'); ylabel('y(t)');
axis equal


% THE RK4 METHOD (COMPLEX)
% We require Re(lambda)<0
[y0,tspan] = odeEq1I;   % standard conditions
lambda=-1+0.5*sqrt(-1); % lambda 
h = 0.2;                % step h 
[tc,yc] = odeRK4('odeEq1',tspan,y0,h,lambda);

% exact solution
t = linspace(tspan(1),tspan(2),100)';
y = odeEq1S(t,lambda);

% draw solution
figure(3);
% real part
subplot(2,1,1)
hfig = plot(t,real(y),'k-',tc,real(yc));
set(hfig,'LineWidth',2);
legend('exact',['h=',num2str(h)]);
title('Real part');
axis equal;
% imaginary part
subplot(2,1,2)
hfig = plot(t,imag(y),'k-',tc,imag(yc));
set(hfig,'LineWidth',2);
legend('exact',['h=',num2str(h)]);
title('imaginary part');
axis equal


% THE ERROR BEHAVIOUR O(h^4)
% The perform error estimation by dividing the step size h by 2 at every
% iteration.
hmax = zeros(10,1);     % Current step size
emax = zeros(10,1);     % Current error
[y0,tspan] = odeEq1I;   % standard conditions
lambda=-1;              % lambda 
h = 0.2;                % step h 

% perform error computation
for k=1:length(hmax)
   [te,ye] = odeRK4('odeEq1',tspan,y0,h,lambda);
   
   y = odeEq1S(te,lambda);
   emax(k)=max(abs(y-ye));
   hmax(k)=h;
   
   h = h/2;
end

% draw error
figure(4);
odeErrorOp(hmax,emax,4);
